/*
  Copyright (c) 2008 Andres Colubri

  This source is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This code is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  A copy of the GNU General Public License is available on the World
  Wide Web at <http://www.gnu.org/copyleft/gpl.html>. You can also
  obtain it by writing to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

// This class implements a brightness curve to control the intensity of an LED.
// The curve is just three linear functions pieced continuously: a line that goes
// from 0 to 1, a constant horizontal line at y = 1, and then another line that goes
// from 1 to 0.
public class BrightCurve
{
    BrightCurve(float d0, float d1, float d2, float ist)
    {
        lightOff = true;
        invTimeStep = ist;
        
        setTimeDeltas(d0, d1, d2);
    }
    
    void setTimeDeltas(float d0, float d1, float d2)
    {
        dinit = d0;
        dmax = d1;
        dfinish = d2;

        initSlope = 1 / dinit; 
        finishSlope = - 1 / dfinish;      
    }
    
    // The light starts at time t.
    void updateHitTime(int t)
    {
        if (lightOff)
        {
            time0 = t;
            lightOff = false;
        }
    }
    
    // Calculates the intensity for time t.
    float calculate(int t)
    {
        if (lightOff)
        {
            return 0;
        }
        else
        {
            float d = invTimeStep * (t - time0);
            
            if (dinit + dmax + dfinish < d)
            {
                // Outside the range of the curve. The light is turned off.
                lightOff = true;
                return 0;
            }
            else if (dinit + dmax < d)
            {
                // In the falloff range (brightness is diminshing towards zero).
                return 1.0 + finishSlope * (d - (dinit + dmax));
            }
            else if (dinit < d)
            {
                // In the constant range. Intensity is maximum.
                return 1.0;
            }
            else
            {
                // Growing region: the intensity is increasing towards the maximum.
                return initSlope * d;
            } 
        }
    }

    private float invTimeStep;
    private float dinit, dmax, dfinish;
    private float initSlope, finishSlope;
    private boolean lightOff;
    private int time0;
}
